<?php
// tòmò mass data generator
// Copyright 2011 by Yves Goergen
// Website: http://unclassified.de/
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
//
// This script generates unrealistic mass data to test the data processing performance.

// ---------- Begin of configuration ----------

// Output SQL file
$outputFile = 'massdata.sql';

// Session time to start generating data for
$firstSessionTime = mktime(0, 0, 0, 1, 1, 2009);   // hour, minute, second, month, day, year
// Session time to stop generating data at
$lastSessionTime = mktime(0, 0, 0, 1, 1, 2010);   // hour, minute, second, month, day, year

// Minimum waiting time between two sessions
$minSessionWaitTime = 10;
// Maximum waiting time between two sessions
$maxSessionWaitTime = 3600;

// Minimum waiting time between two pageviews
$minPageviewWaitTime = 3;
// Maximum waiting time between two pageviews
$maxPageviewWaitTime = 600;

// Minimum number of pageviews per session
$minSessionPageviews = 1;
// Maximum number of pageviews per session
$maxSessionPageviews = 15;

// All available page domains to select from, with assigned probability (free integer value)
$domains = array(
	'example.com' => 1,
	);

// All available page URLs to select from, with assigned probability (free integer value)
$pageUrls = array(
	'/page1' => 10,
	'/page2' => 5,
	'/page3' => 3,
	'/page4' => 1,
	'/page5' => 1,
	);

// Ratio of referring URLs of all sessions
$referrerRatio = 0.2;

// All referrer URLs to select from, with assigned probability (free integer value)
$referrerUrls = array(
	'http://referrer1.com/' => 10,
	'http://referrer2.com/' => 4,
	'http://referrer3.com/' => 1,
	'http://referrer4.com/' => 1,
	);

// Ratio of search engines of all sessions
$searchEngineRatio = 0.7;

// All search engine names to select from, with assigned probability (free integer value)
$searchEngineNames = array(
	'Giga Finder' => 10,
	'Fast Search' => 7,
	'Much Find' => 4,
	'All Find' => 2,
	'Best Search' => 1,
	);

// All search keywords to select from, with assigned probability (free integer value)
$searchKeywords = array(
	'find me' => 10,
	'find this' => 8,
	'find that' => 7,
	'get it all' => 6,
	'get it now!' => 3,
	'tell me everything' => 3,
	'get me out of here' => 3,
	'i am done' => 1,
	);

// Ratio of IPv6 addresses of all sessions
$ipv6Ratio = 0.05;

// All country codes to select from, with assigned probability (free integer value)
$countryCodes = array(
	'DE' => 10,
	'AT' => 5,
	'CH' => 4,
	'FR' => 2,
	'NL' => 1,
	'CZ' => 1,
	'GB' => 1,
	);

// All languages to select from, with assigned probability (free integer value)
$languages = array(
	'de' => 10,
	'en' => 6,
	'fr' => 5,
	'nl' => 4,
	'cs' => 1,
	'hu' => 1,
	);

// ---------- End of configuration ----------

expand_array($domains);
expand_array($pageUrls);
expand_array($referrerUrls);
expand_array($searchEngineNames);
expand_array($searchKeywords);
expand_array($countryCodes);
expand_array($languages);

// text table cache
$textIds = array();

// Outpub buffer
$outputBuffer = '';

$nextSessionId = 1;
$nextVisitorId = 1;
$nextPageviewId = 1;
$nextSessionTime = $firstSessionTime;

unlink($outputFile);

write('begin transaction;');

do
{
	// Start a new session
	$sessionStartTime = gmdate('Y-m-d H:i:s', $nextSessionTime);
	if (mt_rand(0, 1000) <= $referrerRatio * 1000)
	{
		$referrerUrlText = get_random_textid($referrerUrls);
	}
	else
	{
		$referrerUrlText = 'NULL';
	}
	if (mt_rand(0, 1000) <= $searchEngineRatio * 1000)
	{
		$searchEngineText = get_random_textid($searchEngineNames);
		$searchKeywordsText = get_random_textid($searchKeywords);
	}
	else
	{
		$searchEngineText = 'NULL';
		$searchKeywordsText = 'NULL';
	}
	if (mt_rand(0, 1000) <= $ipv6Ratio * 1000)
	{
		$ipVersion = 6;
	}
	else
	{
		$ipVersion = 4;
	}
	$countryCode = get_random_value($countryCodes);
	$language = get_random_value($languages);

	write("insert into visitor (VisitorId) values ($nextVisitorId);");

	write("insert into session (SessionId, VisitorId, NewVisitor, SessionStartTime, SessionDuration, ReferrerHostText, SearchEngineText, SearchKeywordsText, IPAddress, IPVersion, CountryCode, Language) values ($nextSessionId, $nextVisitorId, 1, '$sessionStartTime', 0, $referrerUrlText, $searchEngineText, $searchKeywordsText, '1.2.3.4', $ipVersion, '$countryCode', '$language');");

	// Add some pageviews
	$nextPageviewTime = $nextSessionTime;
	$entryPage = 1;

	$pageviewsCount = mt_rand($minSessionPageview, $maxSessionPageview);
	while ($pageviewsCount--)
	{
		$pageviewTime = gmdate('Y-m-d H:i:s', $nextPageviewTime);
		$domainText = get_random_textid($domains);
		$pageUrlText = get_random_textid($pageUrls);
		$exitPage = !$pageviewsCount ? 1 : 0;

		write("insert into pageview (PageviewId, SessionId, PageviewTime, Type, DomainText, UrlText, Ssl, EntryPage, ExitPage) values ($nextPageviewId, $nextSessionId, '$pageviewTime', 0, $domainText, $pageUrlText, 0, $entryPage, $exitPage);");

		$entryPage = 0;
		$nextPageviewId++;
		$nextPageviewTime += mt_rand($minPageviewWaitTime, $maxPageviewWaitTime);
	}

	$nextSessionId++;
	$nextVisitorId++;
	$nextSessionTime += mt_rand($minSessionWaitTime, $maxSessionWaitTime);
}
while ($nextSessionTime < $lastSessionTime);

write('commit;');
write();   // Flush buffer

// Done! :-)


function write($line = null)
{
	global $outputFile, $outputBuffer;

	if (isset($line))
	{
		$outputBuffer .= $line . "\n";
	}

	if (!isset($line) ||
		strlen($outputBuffer) >= 100000)   // 100 kB
	{
		file_put_contents($outputFile, $outputBuffer, FILE_APPEND);
		$outputBuffer = '';
	}
}

function expand_array(&$array)
{
	$tmp = $array;
	$array = array();
	foreach ($tmp as $value => $prob)
	{
		while ($prob--)
		{
			$array[] = $value;
		}
	}
}

function get_random_value($array)
{
	// Pick a random text from the array
	$text = $array[mt_rand(0, count($array) - 1)];

	return $text;
}

function get_random_textid($array)
{
	global $textIds;

	// Pick a random text from the array
	$text = $array[mt_rand(0, count($array) - 1)];

	// Put it in the text table or get its ID from it
	$textId = $textIds[$text];
	if (!isset($textId))
	{
		$textId = count($textIds);
		$textIds[$text] = $textId;

		write("insert into text (TextId, Value) values ($textId, '$text');");
	}

	return $textId;
}

?>
